// React specific linting rules for ESLint

module.exports =
{
	rules:
	{

		// Specify whether double or single quotes should be used in JSX attributes
		'jsx-quotes': [ 'error', 'prefer-double' ],

		// Enforce that class methods utilize this
		'class-methods-use-this': [ 'warn', {
			exceptMethods: [
				'render',
				'componentWillMount',
				'componentDidMount',
				'componentWillReceiveProps',
				'shouldComponentUpdate',
				'componentWillUpdate',
				'componentDidUpdate',
				'componentWillUnmount'
			]
		} ],

		// Forbid certain propTypes (any, array, object)
		'react/forbid-prop-types': [ 'error', { forbid: [ 'any', 'array', 'object' ] } ],

		// Forbid certain props on Components
		'react/forbid-component-props': [ 'error', { forbid: [ 'className' ] } ],

		// Prevent passing of children as props
		'react/no-children-prop': 'error',

		// Prevent usage of dangerous JSX properties
		'react/no-danger': 'error',

		// Prevent usage of deprecated methods
		'react/no-deprecated': 'error',

		// Prevent usage of setState in componentDidMount
		'react/no-did-mount-set-state': 'warn',

		// Prevent usage of setState in componentDidUpdate
		'react/no-did-update-set-state': 'warn',

		// Prevent direct mutation of this.state
		'react/no-direct-mutation-state': 'error',

		// Prevent usage of isMounted
		'react/no-is-mounted': 'error',

		// Prevent multiple component definition per file
		'react/no-multi-comp': [ 'error', { "ignoreStateless": true} ],

		// Prevent usage of the return value of React.render
		'react/no-render-return-value': 'error',

		// Prevent using string references
		'react/no-string-refs': 'error',

		// Prevent usage of unknown DOM property
		'react/no-unknown-property': [ 1, { ignore: [] } ],

		// Prevent definitions of unused prop types
		'react/no-unused-prop-types': [ 0, {} ],

		// Enforce stateless React Components to be written as a pure function
		'react/prefer-stateless-function': [ 'off' ],

		// Prevent missing props validation in a React component definition
		'react/prop-types': [ 1 ],

		// Prevent missing React when using JSX
		'react/react-in-jsx-scope': 'off',

		// Enforce ES5 or ES6 class for returning value in render function
		'react/require-render-return': 'warn',

		// Prevent extra closing tags for components without children
		'react/self-closing-comp': [ 'warn', {
			'component': true,
			'html': true
		} ],

		// Enforce style prop value being an object
		'react/style-prop-object': 'warn',

		/* JSX-specific rules */

		// Validate closing bracket location in JSX
		'react/jsx-closing-bracket-location': [ 1, {
			selfClosing: 'line-aligned',
			nonEmpty: 'after-props'
		} ],

		// Enforce or disallow spaces inside of curly braces in JSX attributes.
		'react/jsx-curly-spacing': [ 1, 'always', {'spacing': {
			'objectLiterals': 'never'
		} } ],

		// Enforce or disallow spaces around equal signs in JSX attributes.
		'react/jsx-equals-spacing': [ 1, 'never' ],

		// Configure the position of the first property
		'react/jsx-first-prop-new-line': [ 'warn', 'multiline' ],

		// Enforce event handler naming conventions in JSX
		'react/jsx-handler-names': [ 'warn', {
			'eventHandlerPrefix': 'on',
			'eventHandlerPropPrefix': 'on'
		} ],

		// Validate JSX indentation
		'react/jsx-indent': [ 1, 'tab' ],

		// Validate props indentation in JSX
		'react/jsx-indent-props': [ 1, 'tab' ],

		// Detect missing key prop
		'react/jsx-key': 'warn',

		// Limit maximum of props on a single line in JSX
		'react/jsx-max-props-per-line': [ 1, { 'maximum': 1 } ],

		// Prevent comments from being inserted as text nodes
		'react/jsx-no-comment-textnodes': 'warn',

		// Prevent duplicate properties in JSX
		'react/jsx-no-duplicate-props': 'error',

		// Disallow undeclared variables in JSX
		'react/jsx-no-undef': 'error',

		// Enforce PascalCase for user-defined JSX components
		'react/jsx-pascal-case': 'warn',

		// Validate spacing before closing bracket in JSX
		'react/jsx-space-before-closing': [ 'warn', 'always' ],

		// Prevent variables used in JSX to be incorrectly marked as unused
		'react/jsx-uses-vars': 'warn',

		// Prevent missing parentheses around multiline JSX
		'react/jsx-wrap-multilines': [ 1 ]
	},

	// TODO: review each react rule, instead of using this.
	extends: [ 'plugin:react/recommended' ]
};